import React, { useEffect, useState } from 'react'
import { Form, Input, InputNumber, Radio, Modal, Switch } from 'antd';
import positonService from '@/request/service/positon';
import { useRequest } from '@/hooks/use-request';
import type { DepartmentVO } from '@/models'
import { antdUtils } from '@/utils/antd';

const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 },
};

interface EditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: DepartmentVO) => void;
    data?: DepartmentVO | null;
}


const DepartmentEditor: React.FC<EditorProps> = (props) => {

    const { visible, onCancel, onSave, data } = props;

    const { runAsync: updatePostApi } = useRequest(positonService.updatePostApi, { manual: true });
    const { runAsync: createPostApi } = useRequest(positonService.createPostApi, { manual: true });

    const isEdit = !!data;


    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (data) {
                form.setFieldsValue(data);
            }
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const newValue = isEdit ? { ...data, ...fieldValues } : fieldValues;
        const [error, { msg, code }] = isEdit ? await updatePostApi(newValue) : await createPostApi(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                confirmLoading= {saveLoading}
                destroyOnClose
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="name" label="部门名称"
                        rules={[
                            {
                                required: true,
                                message: '请输入岗位名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="phone" label="联系电话"
                        rules={[
                            {
                                required: true,
                                message: '请输入电话号码',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>
                    <Form.Item name="email" label="邮箱"
                        rules={[
                            {
                                required: true,
                                message: '请输入邮箱',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="sort" label="显示排序" >
                        <InputNumber min={1} max={100} />
                    </Form.Item>

                    <Form.Item name="status" label="状态">
                        <Radio.Group options={[
                            { value: 0, label: "开启" },
                            { value: 1, label: "关闭" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
}

export default DepartmentEditor;
