import React, {  useEffect, useState } from 'react'
import { Form, Input, Radio, Modal } from 'antd';
import dictTypeService from '@/request/service/system-dicttype';
import { useRequest } from '@/hooks/use-request';
import type { DictTypeVO } from '@/models'
import { antdUtils } from '@/utils/antd';


const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 }
};

interface DataTypeEditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: DictTypeVO) => void;
    data?: DictTypeVO | null;
}

export default (props: DataTypeEditorProps) => {

    const { visible, onCancel, onSave, data } = props;

    const { runAsync: update } = useRequest(dictTypeService.updateDictTypeApi, { manual: true });
    const { runAsync: create } = useRequest(dictTypeService.createDictTypeApi, { manual: true });

    const isEdit = !!data;


    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (data) {
                form.setFieldsValue(data);
            }
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const newValue = isEdit ? { ...data, ...fieldValues } : fieldValues;
        const [error, { msg, code }] = isEdit ? await update(newValue) : await create(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                confirmLoading= {saveLoading}
                destroyOnClose
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="name" label="字典名称"
                        rules={[
                            {
                                required: true,
                                message: '请输入字典名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="type" label="字典类型"
                        rules={[
                            {
                                required: true,
                                message: '请输入字典类型',
                            },
                        ]}
                    >
                        <Input disabled={isEdit}/>
                    </Form.Item>

                    <Form.Item name="status" label="状态">
                        <Radio.Group options={[
                            { value: 0, label: "开启" },
                            { value: 1, label: "关闭" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>

                    <Form.Item name="remark" label="备注" >
                        <Input.TextArea showCount maxLength={100} />
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
};
